// vim: sw=4:ts=4:nu:nospell:fdc=4
/*global Ext:true */
/*jslint browser: true, devel:true, sloppy: true, white: true, plusplus: true */

/*
 This file is part of saki-grid-search Package

 Copyright (c) 2014, Jozef Sakalos, Saki

 Package:  saki-grid-search
 Author:   Jozef Sakalos, Saki
 Contact:  http://extjs.eu/contact
 Date:     05. May 2014

 Commercial License
 Developer, or the specified number of developers, may use this file in any number
 of projects during the license period in accordance with the license purchased.

 Uses other than including the file in a project are prohibited.
 See http://extjs.eu/licensing for details.
 */

/**
 * Document the file here
 */
Ext.define('Ext.saki.grid.Search',{
     extend:'Ext.AbstractPlugin'
    ,alternateClassName:'Ext.ux.grid.Search'
    ,alias:['plugin.saki-gridsearch', 'plugin.ux-gridsearch']

    ,uses:[
         'Ext.form.field.Text'
    ]

    /**
     * @cfg {String} targetCt Target container selector.
     * {@link Ext.ComponentQuery Ext.ComponentQuery} compatible selector that
     * returns container (usually toolbar) in which to place the search field.
     */
    ,targetCt:'toolbar[dock=bottom]'

    /**
     * @cfg {String} targetCtScope Where to look for the target container. Valid
     * values are 'grid' and 'global'. If the targetCtScope is 'grid' then {@link #targetCt
     * targetCt} is used down from the grid, if it is 'global' then an attempt is made
     * to find the target container anywhere in the application.
     */
    ,targetCtScope:'grid'

    /**
     * @cfg {Boolean} autoCreate If true and {@link #targetCt targetCt container} is not
     * found, the plugin creates toolbar and adds it to the grid according to targetCt
     * dock value (top, right, bottom, left) or bottom if selector does not contain
     * a dock value. The search input is added to the toolbar at the
     * {@link #searchPosition searchPosition}.
     */
    ,autoCreate:true

    /**
     * @cfg {String/Number} searchPosition Search input field position within the
     * {@link #targetCt targetCt container}. Valid values are: first or last to prepend
     * or append the search field to the items of the container. If it is number,
     * the search field is inserted at that index. If it is string other than
     * 'first' or 'last' then the string is used as
     * {@link Ext.ComponentQuery Ext.ComponentQuery} selector to find the container
     * item. If the item is found the search field is places **before** that item.
     * If everything else fails the search field is added as the last item of
     * the container.
     */
    ,searchPosition:'last'

    ,searchAlign:''

    ,fieldsMenu:true

    ,checkIndexes:'all'
    ,disableIndexes:[]
    ,menuStyle:undefined

    /**
     * @cfg {String} inputType Type of the search field. Valid values are: 'search'
     * for <input type="search"> or 'text' for <input type="text">.
     */
    ,inputType:'search'
    ,fontIcons:true


    ,noCtT:'Target container was not found. Check values of '
          +'autoCreate and targetCt config options.'

    ,searchT:'Search'

    ,selectAllT:'Select all'

    ,init:function(cmp) {
        var  me = this
            ,targetCt
        ;

        me.callParent(arguments);

        me.targetCt = targetCt = me.getTargetCt();

        if(!targetCt) {
            Ext.Error.raise(me.noCtT);
        }

        me.disableIndexes = Ext.Array.slice(me.disableIndexes);

        me.createField();

    } // eo function init

    ,reconfigure:function() {
        var  me = this
            ,grid = me.getCmp()
            ,columns = grid.headerCt.items
            ,menu = me.menu
            ,group = 'radio' === me.menuStyle ? 'g' + (new Date()).getTime() : undefined
        ;

        if(menu) {
            menu.removeAll();

            menu.add([{
                 xtype:'menucheckitem'
                ,text:me.selectAllT
                ,checked:'all' === me.checkIndexes
                ,group:group
                ,handler:function(item) {
                    item.parentMenu.items.each(function(mi) {
                        if(mi.setChecked && !mi.isDisabled() && mi !== item) {
                            mi.setChecked(item.checked, true);
                        }
                    });
                }
            },'-']);
            columns.each(function(col) {
                var checked = 'all' === me.checkIndexes || Ext.Array.contains(me.checkIndexes, col.dataIndex);
                if(col.text && col.dataIndex && !Ext.Array.contains(me.disableIndexes, col.dataIndex)) {
                    menu.add({
                         xtype:'menucheckitem'
                        ,text:col.text
                        ,group:group
                        ,checked:checked
                        ,dataIndex:col.dataIndex
                    });
                }
            });
        }
    } // eo function reconfigure

    ,createField:function() {
        var  me = this
            ,targetCt = me.targetCt
            ,position = me.searchPosition
            ,add = me.createAdder()
            ,fieldsMenu = me.fieldsMenu
            ,cfg = []
        ;

        if('last' === me.searchPosition && 'right' === me.searchAlign) {
            cfg.push('->')
        }
        if(true === fieldsMenu) {
            me.menu = Ext.widget({xtype:'menu'});
            cfg.push({
                 xtype:'button'
                ,text:me.searchT
                ,menu:me.menu
            })
        }
        cfg.push({
             xtype:'textfield'
            ,inputType:me.inputType
            ,inputCls:'x-form-text'
            ,isFormField:false
        });

        add(cfg);

        me.reconfigure();

    } // eo function createField

    ,createAdder:function() {
        var  me = this
            ,position = me.searchPosition
            ,targetCt = me.targetCt
            ,beforeItem
            ,beforeItemIndex
        ;
        if(Ext.isNumber(position)) {
            return Ext.Function.bind(targetCt.insert, targetCt, [position], 0);
        }
        if('first' === position) {
            return Ext.Function.bind(targetCt.insert, targetCt, [0], 0);
        }
        if('last' === position) {
            return Ext.Function.bind(targetCt.add, targetCt);
        }
        beforeItem = targetCt.down(position);
        if(beforeItem) {
            beforeItemIndex = targetCt.items.indexOf(beforeItem);
            return Ext.Function.bind(targetCt.insert, targetCt, [beforeItemIndex], 0);
        }
        else {
            return Ext.Function.bind(targetCt.add, targetCt);
        }

    } // eo function createAdder

    ,getTargetCt:function() {
        var  me = this
            ,grid = me.getCmp()
            ,targetCt = 'global' === me.targetCtScope
                ? Ext.ComponentQuery.query(me.targetCt)[0]
                : grid.down(me.targetCt)
            ,position
            ;

        if(!targetCt && me.autoCreate) {
            position = me.targetCt.match(/(top|left|bottom|right)/);
            position = position ? position[0] : 'bottom';

            targetCt = grid.dockedItems.add(
                Ext.widget({xtype:'toolbar', dock:position})
            );
        }

        return targetCt;

    } // eo function getToolbar

}); // eo define

// eof

